"""Configure Sphinx configuration."""
import os
import sys
from datetime import datetime

# Insert the parent directory into the path
sys.path.insert(0, os.path.abspath(".."))

extensions = [
    "myst_parser",
]
templates_path = ["_templates"]
source_suffix = ".rst"
master_doc = "index"

project = "nws-aurora"
year = datetime.now().year
copyright = f"{year} Ben Welsh"

exclude_patterns = ["_build"]

html_theme = "alabaster"
html_sidebars = {
    "**": [
        # "about.html",
        # "navigation.html",
        "relations.html",
        "searchbox.html",
        "donate.html",
    ]
}
html_theme_options = {
    "canonical_url": f"https://palewi.re/docs/{project}/",
    "show_powered_by": False,
    "show_relbar_bottom": True,
}

html_static_path = ["_static"]
html_css_files = [
    "css/custom.css",
]

pygments_style = "sphinx"



import re
from datetime import datetime

import pytz
import requests
from bs4 import BeautifulSoup
from geojson import Feature, FeatureCollection, Point

IMAGE_REGEX = re.compile("aurora_[N|S]_(.*).jpg")


def _parse_image_date(s):
    s = re.findall(IMAGE_REGEX, s)[0]
    d = datetime.strptime(s, "%Y-%m-%d_%H%M").replace(tzinfo=pytz.UTC)
    return d.isoformat()


def _parse_forecast_date(s):
    d = datetime.strptime(s, "%Y-%m-%d_%H:%M").replace(tzinfo=pytz.UTC)
    return d.isoformat()


def get_images(pole):
    """
    Get URLs to all of the OVATION model images for the last 24 hours for the provided pole.

    Returns list of dictionaries
    """
    url = f"https://services.swpc.noaa.gov/images/animations/ovation/{pole}/"
    r = requests.get(url)
    s = BeautifulSoup(r.content, "html.parser")
    a_list = s.find_all("a")
    href_list = []
    for a in a_list:
        if "latest.jpg" in a["href"]:
            continue
        if ".jpg" not in a["href"]:
            continue
        href_list.append(
            dict(
                url=f"{url}{a['href']}",
                timestamp=_parse_image_date(a["href"]),
                pole=pole,
            )
        )
    return href_list


def get_latest_image(pole):
    """
    Get URL to the latest OVATION model image for the provided poll.

    Returns a URL string.
    """
    return f"https://services.swpc.noaa.gov/images/animations/ovation/{pole}/latest.jpg"


def get_grid():
    """
    Get auroral data in a gridded format for the entire Earth.

    Returns GeoJSON.
    """
    r = requests.get("https://services.swpc.noaa.gov/json/ovation_aurora_latest.json")
    j = r.json()
    feature_list = []
    for d in j["coordinates"]:
        f = Feature(geometry=Point((d[0], d[1])), properties=dict(aurora=d[2]))
        feature_list.append(f)
    fc = FeatureCollection(feature_list)
    fc["properties"] = dict(
        observation_time=j["Observation Time"], forecast_time=j["Forecast Time"]
    )
    return fc


def get_forecast():
    """
    Get Ovation Aurora Short Term Forecast data.

    Returns a list of dictionaries
    """
    r = requests.get(
        "https://services.swpc.noaa.gov/text/aurora-nowcast-hemi-power.txt"
    )
    t = r.text.splitlines()
    str_list = t[16:]
    row_list = []
    for s in str_list:
        val_list = [v.strip() for v in s.split("    ")]
        d = dict(
            observation_time=_parse_forecast_date(val_list[0]),
            forecast_time=_parse_forecast_date(val_list[1]),
            hpi_north=int(val_list[2]),
            hpi_south=int(val_list[3]),
        )
        row_list.append(d)
    return row_list



import click

from nws_aurora import get_forecast, get_grid, get_images, get_latest_image


@click.group()
def cmd():
    """
    A command-line interface for downloading forecast data for Aurora Borealis and Aurora Australis
    from the National Weather Service
    """
    pass


@cmd.command(help="Ovation model images from the last 24 hours")
@click.option("--latest", is_flag=True)
@click.option("--pole", required=True, type=click.Choice(["north", "south"]))
def images(pole, latest):
    if latest:
        click.echo(get_latest_image(pole))
    else:
        click.echo(get_images(pole))


@cmd.command(help="Get auroral data in a gridded format for the entire Earth")
def grid():
    click.echo(get_grid())


@cmd.command(help="Get Ovation Aurora Short Term Forecast data")
def forecast():
    click.echo(get_forecast())



import os

from setuptools import setup


def read(file_name):
    this_dir = os.path.dirname(__file__)
    file_path = os.path.join(this_dir, file_name)
    with open(file_path) as f:
        return f.read()


def version_scheme(version):
    """
    Version scheme hack for setuptools_scm.

    Appears to be necessary to due to the bug documented here: https://github.com/pypa/setuptools_scm/issues/342

    If that issue is resolved, this method can be removed.
    """
    import time

    from setuptools_scm.version import guess_next_version

    if version.exact:
        return version.format_with("{tag}")
    else:
        _super_value = version.format_next_version(guess_next_version)
        now = int(time.time())
        return _super_value + str(now)


def local_version(version):
    """
    Local version scheme hack for setuptools_scm.

    Appears to be necessary to due to the bug documented here: https://github.com/pypa/setuptools_scm/issues/342

    If that issue is resolved, this method can be removed.
    """
    return ""


setup(
    name="nws-aurora",
    description="Download forecast data for Aurora Borealis and Aurora Australis from the National Weather Service",
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    author="Ben Welsh",
    author_email="b@palewi.re",
    url="https://palewi.re/docs/nws-aurora/",
    license="MIT",
    packages=("nws_aurora",),
    entry_points="""
        [console_scripts]
        nwsaurora=nws_aurora.cli:cmd
    """,
    use_scm_version={"version_scheme": version_scheme, "local_scheme": local_version},
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "License :: OSI Approved :: MIT License",
    ],
    project_urls={
        "Maintainer": "https://github.com/palewire",
        "Source": "https://github.com/palewire/nws-aurora",
        "Tracker": "https://github.com/palewire/nws-aurora/issues",
    },
)



#! /usr/bin/env python
import unittest

from nws_aurora import get_forecast, get_grid, get_images, get_latest_image


class NwsAuroraTest(unittest.TestCase):
    def test_images(self):
        get_images("north")
        get_images("south")

    def test_latest_image(self):
        get_latest_image("north")
        get_latest_image("south")

    def test_grid(self):
        get_grid()

    def test_forecast(self):
        get_forecast()


if __name__ == "__main__":
    unittest.main()
